#=======================================================================
#	$Id: Conv.pm,v 1.4 2006/05/18 10:47:48 pythontech Exp $
#	Conversions to (un)escape text
#	Copyright (C) 2005  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#=======================================================================
package PythonTech::Conv;
use strict;
use Exporter;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(html_escape html_unescape
		    hq hx
		    xml_escape xml_unescape
		    xq xx
		    uri_escape uri_unescape
		    uq ux
		    c_escape c_unescape
		    cq cx);

#-----------------------------------------------------------------------
#	Escape arbitrary text for use as part of a URI, e.g. the
#	value of a query parameter.
#-----------------------------------------------------------------------
sub uri_escape {
    my($text) = @_;
    $text =~ s/([^A-Za-z0-9_\.])/sprintf("%%%02X",unpack("C",$1))/eg;
    return $text;
}

sub uri_unescape {
    my($text) = @_;
    $text =~ s/%([0-9a-fA-F][0-9a-fA-F])/pack("C", hex($1))/eg;
    return $text;
}

#-----------------------------------------------------------------------
#	Excape arbitrary text so that it can be included in HTML
#-----------------------------------------------------------------------
sub html_escape {
    my($text) = @_;
    $text =~ s/\&/\&amp;/g;
    $text =~ s/\</\&lt;/g;
    $text =~ s/\>/\&gt;/g;
    $text =~ s/\"/\&quot;/g;
    return $text;
}

sub html_unescape {
    my($html) = @_;
    my $text = '';
    while (1) {
	if ($html =~ m!\G([^\&]+)!gc) {
	    $text .= $1;
	} elsif ($html =~ m!\G\&([a-z]+);!gc) {
	    my $ch = {
		amp => '&',
		lt => '<',
		gt => '>',
		quot => '"',
	    }->{$1};
	    die "Unknown entity \&$1\n"
		unless defined $ch;
	    $text .= $ch;
	} elsif ($html =~ m!\G\&\#(\d+);!gc) {
	    $text .= pack("C", $1);
	} else {
	    last;
	}
    }
    die "Unrecognised html escape at ".substr($html,pos($html),10)."\n"
	unless pos($html) == length($html);
    return $text;
}

#-----------------------------------------------------------------------
#	Excape arbitrary text so that it can be included in XML
#-----------------------------------------------------------------------
sub xml_escape {
    my($text) = @_;
    $text =~ s/\&/\&amp;/g;
    $text =~ s/\</\&lt;/g;
    $text =~ s/\>/\&gt;/g;
    $text =~ s/\"/\&quot;/g;
    $text =~ s/\'/\&apos;/g;
    return $text;
}

sub xml_unescape {
    my($xml) = @_;
    my $text = '';
    while (1) {
	if ($xml =~ m!\G([^\&]+)!gc) {
	    $text .= $1;
	} elsif ($xml =~ m!\G\&([a-z]+);!gc) {
	    my $ch = {
		amp => '&',
		lt => '<',
		gt => '>',
		quot => '"',
		apos => "'",
	    }->{$1};
	    die "Unknown entity \&$1\n"
		unless defined $ch;
	    $text .= $ch;
	} elsif ($xml =~ m!\G\&\#(\d+);!gc) {
	    $text .= pack("C", $1);
	} else {
	    last;
	}
    }
    die "Unrecognised xml escape at ".substr($xml,pos($xml),10)."\n"
	unless pos($xml) == length($xml);
    return $text;
}

#-----------------------------------------------------------------------
#	Escape arbitrary text with common codes compatible with C, Perl:
#	\t \n \r \\ \" \012 \xa3
#	
#-----------------------------------------------------------------------
sub c_escape {
    my($text) = @_;
    $text =~ s!\\!\\\\!g;
    $text =~ s!\"!\\\"!g;
    $text =~ s!\t!\\t!g;
    $text =~ s!\n!\\n!g;
    $text =~ s!\r!\\r!g;
    $text =~ s!([^\x20-\x7e])!sprintf("\\x%02x",unpack("C",$1))!eg;
    return $text;
}

sub c_unescape {
    my($text) = @_;
    my $res;
    while (1) {
	if ($text =~ m!\G([^\\]+)!gc) {
	    $res .= $1;
	} elsif ($text =~ m!\G\\([tnrfbae])!gc) {
	    $res .= {
		t=>"\t",
		n=>"\n",
		r=>"\r",
		f=>"\f",
		b=>"\b",
		a=>"\a",
	    }->{$1};
	} elsif ($text =~ m!\G\\x([0-9a-f]{2})!gci) {
	    $res .= pack("C", hex($1));
	} elsif ($text =~ m!\G\\([0-7]{1,3})!gc) {
	    $res .= pack("C", oct($1));
	} elsif ($text =~ m!\G\\([\\\"])!gc) {
	    $res .= $1;
	} else {
	    last;
	}
    }
    die "Unrecognised c escape at ".substr($text,pos($text),10)."\n"
	unless pos($text) == length($text);
    return $res;
}

#-----------------------------------------------------------------------
#	Aliases
#-----------------------------------------------------------------------
sub hq {html_escape(@_)}
sub hx {html_unescape(@_)}
sub xq {xml_escape(@_)}
sub xx {xml_unescape(@_)}
sub uq {uri_escape(@_)}
sub ux {uri_unescape(@_)}
sub cq {c_escape(@_)}
sub cx {c_unescape(@_)}

1;
